#include "term.h"

static inline uint8_t vga_entry_color(enum vga_color fg, enum vga_color bg) {
	return fg | bg << 4;
}


static inline uint16_t vga_entry(unsigned char uc, uint8_t color) {
	return (uint16_t) uc | (uint16_t) color << 8;
}


void terminal_initialize(kVGATerm *term) {
	term->terminal_row = 0;
	term->terminal_column = 0;
	term->terminal_color = vga_entry_color(VGA_COLOR_LIGHT_GREY, VGA_COLOR_BLACK);
	term->terminal_buffer = (uint16_t*) 0xB8000;

	for (size_t y = 0; y < VGA_HEIGHT; y++) {
		for (size_t x = 0; x < VGA_WIDTH; x++) {
			const size_t index = y * VGA_WIDTH + x;
			term->terminal_buffer[index] = vga_entry(' ', term->terminal_color);
		}
	}
}

void terminal_shiftup(kVGATerm *term){
	for(size_t y = 0; y < VGA_HEIGHT-1; y++) {
		for (size_t x = 0; x < VGA_WIDTH; x++) {
			size_t index_current = y * VGA_WIDTH  + x;
			size_t index_next = (y+1) * VGA_WIDTH  + x;
			term->terminal_buffer[index_current] =  term->terminal_buffer[index_next];
		}
	}
}

void terminal_setcolor(kVGATerm *term, uint8_t color){
	term->terminal_color = color;
}

void terminal_putentryat(kVGATerm *term, char c, uint8_t color, size_t x, size_t y){
	size_t index = y * VGA_WIDTH + x;
	term->terminal_buffer[index] = vga_entry(c, color);
}

void terminal_putchar(kVGATerm *term, char c){
	if (c == '\n'){
		term->terminal_row = term->terminal_row + 1;
		term->terminal_column = 0;
		return;
	}
	terminal_putentryat(term, c, term->terminal_color, term->terminal_column, term->terminal_row);
	term->terminal_column = term->terminal_column + 1;
	if (term->terminal_column >= VGA_WIDTH) {
		term->terminal_column = 0;
		term->terminal_row = term->terminal_row + 1;
	}
	if (term->terminal_row >= VGA_HEIGHT) {
	terminal_shiftup(term);
	//terminal_row = 0;
	}
}

void terminal_write(kVGATerm *term, const char* data, size_t size) {
	for (size_t i = 0; i < size; i++) {
		terminal_putchar(term, data[i]);
	}

}

void terminal_writestring(kVGATerm *term, const char* data) {
	terminal_write(term, data, strlen(data));
}

size_t strlen(const char* str) {
	size_t len = 0;
	while (str[len])
		len++;
	return len;
}
